@@ -64,20 +64,23 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 /*----------------------------------------------------------------------*/
 
 /* Single-bit flags */
-#define	OBJINST		1
-#define LABEL		2
-#define POLYGON		4
-#define ARC		8
-#define SPLINE  	16
-#define PATH		32
+#define	OBJINST		0x01
+#define LABEL		0x02
+#define POLYGON		0x04
+#define ARC		0x08
+#define SPLINE  	0x10
+#define PATH		0x20
+#define GRAPHIC		0x40	/* reserved; unused, for now.		*/
+#define ARRAY		0x80	/* reserved; unused, for now.		*/
 
-#define REMOVE_TAG	64	/* element to be removed from netlist	*/
-#define NETLIST_INVALID 128	/* this element invalidates the netlist	*/
-#define SELECT_HIDE	256	/* this element cannot be selected	*/
-#define DRAW_HIDE	512	/* this element is not drawn.		*/
+#define REMOVE_TAG	0x100	/* element to be removed from netlist	*/
+#define NETLIST_INVALID 0x200	/* this element invalidates the netlist	*/
+#define SELECT_HIDE	0x400	/* this element cannot be selected	*/
+#define DRAW_HIDE	0x800	/* this element is not drawn.		*/
 
 /* Bit fields */
-#define ALL_TYPES 	(OBJINST | LABEL | POLYGON | ARC | SPLINE | PATH)
+#define ALL_TYPES 	(OBJINST | LABEL | POLYGON | ARC | SPLINE | PATH \
+				| GRAPHIC | ARRAY)
 #define VALID_TYPES 	(REMOVE_TAG | ALL_TYPES)
 
 /*----------------------------------------------------------------------*/
@@ -119,6 +122,8 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define IS_ARC(a)       (ELEMENTTYPE(a) == ARC)
 #define IS_SPLINE(a)    (ELEMENTTYPE(a) == SPLINE)
 #define IS_PATH(a)      (ELEMENTTYPE(a) == PATH)
+#define IS_GRAPHIC(a)   (ELEMENTTYPE(a) == GRAPHIC)
+#define IS_ARRAY(a)     (ELEMENTTYPE(a) == ARRAY)
 
 /* Conversions from generic to specific types */
 /* specifically, from type (genericptr *) to type (polyptr), etc. */
@@ -129,6 +134,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define TOARC(a)	(*((arcptr *)(a)))
 #define TOSPLINE(a)	(*((splineptr *)(a))) 
 #define TOPATH(a)	(*((pathptr *)(a)))
+#define TOGRAPHIC(a)	(*((graphicptr *)(a)))
 
 /* conversions from a selection to a specific type */
 
@@ -140,6 +146,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define SELTOARC(a)	TOARC(topobject->plist + (*(a)))
 #define SELTOSPLINE(a)	TOSPLINE(topobject->plist + (*(a)))
 #define SELTOPATH(a)	TOPATH(topobject->plist + (*(a)))
+#define SELTOGRAPHIC(a)	TOGRAPHIC(topobject->plist + (*(a)))
 #define SELTOGENERIC(a) ((areastruct.drawstack != NULL) ? \
 			(genericptr)areastruct.drawstack->thisinst : \
 			(*(topobject->plist + (*(a)))))
@@ -176,22 +183,11 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	a = (pathptr *)b->plist + b->parts; \
 	*a = (pathptr) malloc(sizeof(path)); \
 	(*a)->type = PATH
-
-/*----------------------------------------------------------------------*/
-/* Undo mechanism definitions						*/
-/*----------------------------------------------------------------------*/
-
-enum UNDO_MODES {UNDO_DONE = 0, UNDO_MORE};
-
-/*----------------------------------------------------------------------*/
-/* selection from top-level object 					*/
-/*----------------------------------------------------------------------*/
-
-#define select_element(a) genselectelement(a, UNDO_MORE, topobject,\
-	areastruct.topinstance)
-#define select_add_element(a) genselectelement(a, UNDO_DONE, topobject,\
-	areastruct.topinstance)
-#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
+#define NEW_GRAPHIC(a,b) \
+	PLIST_INCR(b); \
+	a = (graphicptr *)b->plist + b->parts; \
+	*a = (graphicptr) malloc(sizeof(graphic)); \
+	(*a)->type = GRAPHIC
 
 /*----------------------------------------------------------------------*/
 
@@ -609,6 +605,21 @@ typedef struct _liblist {
    liblistptr	next;
 } liblist;
 
+/*----------------------------------------------------------------------*/
+/* Generalized graphic object (Tcl/Tk only)				*/
+/*----------------------------------------------------------------------*/
+
+typedef struct {
+   u_short	  type;
+   int		  color;	/* foreground, for bitmaps only */
+   eparamptr	  passed;	/* numerical parameters passed from above */
+   XPoint	  position;
+   short	  rotation;
+   float	  scale;
+   Pixmap	  source;	/* source data */
+   Pixmap	  target;	/* target (scaled) data */
+} graphic, *graphicptr;
+
 /*----------------------------------------------------------------------*/
 /* Label								*/
 /*----------------------------------------------------------------------*/
@@ -688,6 +699,31 @@ typedef struct {
    genericptr	*plist;		/* to be retypecast to polygon, arc, or spline */
 } path, *pathptr;
 
+/*----------------------------------------------------------------------*/
+/* selection from top-level object 					*/
+/*----------------------------------------------------------------------*/
+
+/*----------------------------------------------------------------------*/
+/* Undo mechanism definitions						*/
+/*----------------------------------------------------------------------*/
+
+enum UNDO_MODES {UNDO_DONE = 0, UNDO_MORE, MODE_CONNECT, MODE_RECURSE_WIDE,
+	MODE_RECURSE_NARROW};
+
+typedef struct _selection *selectionptr;
+
+typedef struct _selection {
+   int selects;
+   short *selectlist;
+   objinstptr thisinst;
+   selectionptr next;
+} selection;
+
+#define select_element(a) recurse_select_element(a, UNDO_MORE) 
+#define select_add_element(a) recurse_select_element(a, UNDO_DONE) 
+
+#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
+
 /*----------------------------------------------------------------------*/
 /* Netlist structures for schematic capture				*/
 /*----------------------------------------------------------------------*/
@@ -1104,6 +1140,7 @@ typedef struct {
    /* per-drawing-page parameters */
    objinstptr	pageinst;
    char		*filename;	/* file to save as */
+   u_char	idx;		/* page index */
    psbkground	background;	/* background rendered file info */
    float	wirewidth;
    float	outscale;
@@ -1216,8 +1253,8 @@ typedef struct {
    xcIntervalId	timeout_id;
    int		save_interval;
    u_short	new_changes;
-   short	pages;
    short	numlibs;
+   short	pages;
    Pagedata	**pagelist;
    Undoptr	undostack;	/* actions to undo */
    Undoptr	redostack;	/* actions to redo */
